@@ -1,8 +1,17 @@
 def server():
+    import argparse
+
+    parser = argparse.ArgumentParser()
+    parser.add_argument("-r", "--reuse", action="store_true", help="Allow reuse of the server socket by setting SO_REUSEADDR")
+    args = parser.parse_args()
+
     import asyncio
     import spoke
 
-    server = spoke.pubsub.server.Server()
+    conn_opts = {
+        "reuse": args.reuse,
+    }
+    server = spoke.pubsub.server.Server(conn_opts=conn_opts)
 
     try:
         asyncio.run(server.run())
@@ -27,13 +36,16 @@ def echo():
     parser.add_argument(
         "--label", "-l", help="Label to include at beginning of each output"
     )
-    parser.add_argument("--verbose", "-v", help="Print the full message head and body")
+    parser.add_argument(
+        "--verbose",
+        "-v",
+        action="store_true",
+        help="Print the full message head and body",
+    )
     parser.add_argument("channel", nargs="*", default=["**"])
     args = parser.parse_args()
 
-    import sys
     import asyncio
-    import atexit
     import spoke
     import signal
 
@@ -63,7 +75,7 @@ def echo():
         await asyncio.gather(
             *[client.subscribe(x, echo, bounce=False) for x in args.channel]
         )
-        await spoke.wait.wait()
+        await spoke.wait()
         if ostream:
             ostream.close()
 
@@ -82,7 +94,6 @@ def call():
 
     import json
     import spoke
-    import sys
 
     if args.body is None:
         body = args.body
@@ -90,14 +101,14 @@ def call():
         body = json.loads(args.body)
 
     try:
-        print(spoke.simple.call(args.channel, body, timeout=args.timeout))
+        print(spoke.call(args.channel, body, timeout=args.timeout))
     except TimeoutError:
         print("Timeout")
-        sys.exit(1)
+        return 1
     except spoke.pubsub.error.RemoteCallError as e:
         msg = "Error: {}"
         print(msg.format(e))
-        sys.exit(1)
+        return 1
     return 0
 
 
@@ -119,7 +130,7 @@ def publish():
     else:
         body = json.loads(args.body)
 
-    spoke.simple.publish(args.channel, body, timeout=args.timeout, persist=args.persist)
+    spoke.publish(args.channel, body, timeout=args.timeout, persist=args.persist)
     return 0
 
 
@@ -134,30 +145,43 @@ def bridge():
     args = parser.parse_args()
 
     import asyncio
-    import sys
     import spoke
 
     if args.port1 == args.port2 and args.host1 == args.host2:
         print("The given arguments will bridge the server to itself. Aborting.")
-        sys.exit(1)
+        return 1
+
+    state = {}
+
+    async def on_connect(conn):
+        for msg in state.values():
+            await conn.send(msg)
 
     def mirror(other):
         async def _inner(msg):
             if msg.channel.startswith("-control/"):
                 return
             msg.head["bounce"] = False
+            if msg.head.get("persist", True):
+                state[msg.channel] = msg
             await other.publish(body=msg.body, **msg.head)
 
         return _inner
 
     async def main():
-        client1 = spoke.pubsub.client.Client(port=args.port1, host=args.host1)
+        client1 = spoke.pubsub.client.Client(
+            conn_opts={"port": args.port1, "host": args.host1},
+            on_connect=on_connect,
+        )
         await client1.run()
-        client2 = spoke.pubsub.client.Client(port=args.port2, host=args.host2)
+        client2 = spoke.pubsub.client.Client(
+            conn_opts={"port": args.port2, "host": args.host2},
+            on_connect=on_connect,
+        )
         await client2.run()
-        await client1.subscribe("**", mirror(client2))
-        await client2.subscribe("**", mirror(client1))
-        await spoke.wait.wait()
+        await client1.subscribe("**", mirror(client2), bounce=False)
+        await client2.subscribe("**", mirror(client1), bounce=False)
+        await spoke.wait()
 
     asyncio.run(main())
     return 0
